
// Version 1.2.1.1 mqe_db2_adapter_supportpac/source/examples/db2/ConfigInfo.java, MQeDB2Adapter, la202

/*-----------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms               *
 *                                                                             *
 * Licensed Materials - Property of IBM                                        *
 *                                                                             *
 * 5765-E63                                                                    *
 *                                                                             *
 * (C) Copyright IBM Corp. 2002                                                *
 *-----------------------------------------------------------------------------*/

package examples.db2
;
import com.ibm.mqe.*;
import com.ibm.mqe.registry.*;
import com.ibm.mqe.adapters.*;

/**
 * Holds configuration information about the db2-resident queue manager centrally so that
 * the configure, run and unconfigure classes all use the same names for things.
 *
 * <p>Use the activate method first, then getters.
 */
public class ConfigInfo {

	public static short[] version = { 2, 0, 0, 0 };

	/**
	 * The name of the queue manager the examples in this package use.
	 */
	private String queueManagerName ;

	/**
	 * Returns the name of the queue manager to use.
	 */
	public String getQueueManagerName() {
		return queueManagerName;
	}

	/**
	 * The name of the base directory that the examples in this package use.
	 */
    private String baseDirectoryName ;

    public String getBaseDirectoryName() {
    	return this.baseDirectoryName;
    }

    /**
     * The name of the test queue to which a test message is put by programs in this package.
     */
    private String testQueueName ;

    public String getTestQueueName() {
    	return testQueueName ;
    }

	/**
	 * An MQeFields structure which will describe the parameters required to start the WMQe queue manager.
	 */
	private MQeFields config = null ;

	/**
	 * Retrieves a set of MQeFields which can be used to activate a queue manager.
	 * <p>It gives the queue manager enough information so that it can load-up its' configuration data and
	 * start to act on the message data contained in existing queues.
	 */
	public MQeFields getConfigFields() {
		return config ;
	}

    private String testQueueStorageLocation ;

	/**
	 * Returns the testQueue1StorageLocation.
	 * @return String
	 */
	public String getTestQueueStorageLocation() {
		return testQueueStorageLocation;
	}

	/**
	 * Private reference to the storage location which the system queues should use.
	 */
    private String systemQueueStorageLocation ;

    /**
	 * Returns the systemQueueStorageLocation.
	 * @return String
	 */
	public String getSystemQueueStorageLocation() {
		return systemQueueStorageLocation;
	}

	/**
	 * Max message size of a single dumped message on the test queues.
	 */
	private int maxQueueMessageSizeKBytes = 16 ;

	/**
	 * Returns the max size each message should be.
	 */
	public int getMaxQueueMessageSizeKBytes() {
		return this.maxQueueMessageSizeKBytes ;
	}

    /**
     * Change this flag if your want to gather as much debugging information as possible from the
     * MQeMappingAdapter
     */
    private static final boolean DEBUG_MAPPING_ADAPTER =false ;

    public void activate() throws Exception {
    	String registryStorageAdapter ;

		// Using a short queue manager name helps keep a message files' fully
    	// qualified name short.
    	queueManagerName = "QM1";
    	testQueueName = "MyTestQueue";

    	String fileSeparator =  MQe.fileSeparator();

		// Set the base directory in which queues will deposit their messages.
		baseDirectoryName = "C:"+fileSeparator+"QM1";

		systemQueueStorageLocation = "com.ibm.mqe.adapters.MQeJDBCFieldsAdapter" + ":" + baseDirectoryName + MQe.fileSeparator() + "Queues" ;
		testQueueStorageLocation = "com.ibm.mqe.adapters.MQeJDBCFieldsAdapter" + ":" + baseDirectoryName + MQe.fileSeparator() + "Queues"+MQe.fileSeparator()+testQueueName;
		registryStorageAdapter = "com.ibm.mqe.adapters.MQeDiskFieldsAdapter" ;

        // Create all the configuration information needed to construct the
    	// queue manager in memory.
     	config = new MQeFields();

        // Construct the queue manager section parameters.
        MQeFields queueManagerSection = new MQeFields();

        queueManagerSection.putAscii(MQeQueueManager.Name, queueManagerName);
        config.putFields(MQeQueueManager.QueueManager, queueManagerSection);

        // Construct the registry section parameters.
        // In this examples, we use a public registry.
        MQeFields registrySection = new MQeFields();

		// Make sure the registry sits on the chosen adapter
        registrySection.putAscii(MQeRegistry.Adapter, registryStorageAdapter );
        registrySection.putAscii(MQeRegistry.DirName, baseDirectoryName+MQe.fileSeparator()+"Registry");
        config.putFields("Registry", registrySection);


    }
}
