// Version 1.1.1.1 mqe_db2_adapter_supportpac/source/examples/db2/Configure.java, MQeDB2Adapter, la202

/*-----------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms               *
 *                                                                             *
 * Licensed Materials - Property of IBM                                        *
 *                                                                             *
 * 5765-E63                                                                    *
 *                                                                             *
 * (C) Copyright IBM Corp. 2002                                                *
 *-----------------------------------------------------------------------------*/
package examples.db2;

import examples.mqbridge.administration.programming.*;
import com.ibm.mqe.*;

/**
 * Configures a queue manager on your disk, such that configuration and message store 
 * information is held in DB2 tables.
 * 
 * <p>It uses the MQeDB2FieldsAdapter adapter class to handle connections to DB2.
 * 
 * <p>The resultant queue manager can then be used by the 
 * {@link examples.db2.Run} program, and destroyed using the 
 * {@link examples.db2.UnConfigure} program.
 * 
 * <p>Most solutions separate the tasks of deployment and configuratation their queue managers
 * from the use of those queue managers. This example does this also.
 */
public class Configure {
	
    public static short[] version = { 2, 0, 0, 0 };
    
	/**
	 * Indicates whether this program puts MQe trace to stdout or not.
	 */
	private static final boolean MQE_TRACE = false ;

    /**
     * Entry point invoked from the command line.
     */
    public static void main(String[] args) {
        try {
        	TraceController traceHelper = new TraceController();
     		if( MQE_TRACE ) {
        		traceHelper.traceOn();
     		}
        	
        	Configure me = new Configure();
            me.define();
            me.defineTestQueue();
            
            if( MQE_TRACE ) {
            	traceHelper.traceOff();
            }
        } catch (Exception e) {
            System.out.println("Failure: " + e.toString());
            e.printStackTrace();
        }	
    }
    
	/**
     * Defines the queue manager. And creates a test queue on the queue manager for use later.
     * 
     * <p>All queues defined have names which are independent of the underlying 8.3 file system
     * on which they depend. Each queue is set up with an explicit directory in which messages are stored. 
     */
    public void define() throws Exception {    	
    	
		ConfigInfo configInfo = new ConfigInfo();
		configInfo.activate();
		
		MQeFields configFields = configInfo.getConfigFields();
		String systemQueueStorageLocation = configInfo.getSystemQueueStorageLocation();
		String queueManagerName = configInfo.getQueueManagerName();
		String fileSeparator = MQe.fileSeparator();
		
        System.out.println("Defining the queue manager.");
				
        // Construct a queue manager configuration utility object.
        MQeQueueManagerConfigure configurator = new MQeQueueManagerConfigure( configFields , systemQueueStorageLocation );
        configurator.setQueueStore( systemQueueStorageLocation );

        // Define a queue manager.				
        configurator.defineQueueManager();

        // Define some queues on the queue manager.  
        // Explicitly stating where their storage locations are based.
        configurator.defineDefaultAdminQueue( "My Admin Queue" );		  									
        configurator.defineDefaultAdminReplyQueue( "My Admin Reply Queue" );        															  
        configurator.defineDefaultDeadLetterQueue( "My Dead Letter Queue" );
        configurator.defineDefaultSystemQueue( "My Default System Queue" );	
						
        // Close the queue manager configuration utility class.
        configurator.close();
        System.out.println("Defined queue manager " + queueManagerName );
    }
    
    /**
     * Define a test queue using administration messages, so we can put a test message to it.
     * 
     * <p>We could have used the system default local queue, but here we are showing an example
     * of how one might create a queue which has a long filename itself, but stores messages
     * on an 8.3 file sysyem. 
     * 
     * @throws Exception if there is a failure.
     */
    public void defineTestQueue() throws Exception {
    	System.out.println("Creating a test queues");
    	
    	ConfigInfo configInfo = new ConfigInfo();
		configInfo.activate();
		
		MQeFields configFields = configInfo.getConfigFields();
		String qStorageLocation = configInfo.getTestQueueStorageLocation();
		String queueManagerName = configInfo.getQueueManagerName();
		String fileSeparator = MQe.fileSeparator();
		String testQueueName = configInfo.getTestQueueName();

    	
        // Now fire-up the queue manager so we can adminster it using adminstration messages...
        System.out.println("Starting the queue manager so we can administer it.");
        MQeQueueManager myQueueManager = new MQeQueueManager();
        myQueueManager.activate(configFields);
        System.out.println("Queue manager started. We can now administer it.");
        
        // Construct a test queue explicitly which stores 8.3 message fies in a short filename directory.
		Reporter reporter = (Reporter) new AdminHelperReporter();
		AdminMessageDispatcher dispatcher = (AdminMessageDispatcher)new AdminHelperDispatcher(myQueueManager);
		AdminHelperLocalQueue helper = new AdminHelperLocalQueue( reporter , dispatcher );
		  
        try {
        	try {
	        	System.out.println("Test queue name is "+testQueueName);
	        	System.out.println("Test queue will store messages to "+qStorageLocation);
				helper.create(  queueManagerName , testQueueName , qStorageLocation );
	        } catch( Exception e ) {
	        	reporter.error("Failed to create the test queue "+testQueueName);        	
	        	reporter.error(e);
        	}
        	
        } finally {
        	// Even if we failed, make sure the queue manager is closed down within 5 seconds.
        	System.out.println("Stopping the queue manager.");
        	myQueueManager.closeQuiesce(5000L);
        	System.out.println("Stopped the queue manager.");
        }		
    }	    	   
}
