// Version 1.2.1.1 mqe_db2_adapter_supportpac/source/examples/db2/PerformanceTest.java, MQeDB2Adapter, la202

/*-----------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms               *
 *                                                                             *
 * Licensed Materials - Property of IBM                                        *
 *                                                                             *
 * 5765-E63                                                                    *
 *                                                                             *
 * (C) Copyright IBM Corp. 2002                                                *
 *-----------------------------------------------------------------------------*/
package examples.db2;


import com.ibm.mqe.*;
import com.ibm.mqe.adapters.*;
import com.ibm.mqe.registry.*;
import com.ibm.mqe.trace.MQeTraceToReadable;
import java.util.*;
import java.io.*;

import examples.mqbridge.administration.programming.*;


/**
 * Uses the configured queue manager previously defined by the {@link examples.db2.Configure} program.
 * 
 * <p>3 queues are created. One for each of the DB2, disk and memory adapters.
 * A sequence of messages are put to the test queues, and got back from the test queues.
 * This is done for both assured and non-assured operations. The results are timed, and reported
 * on stdout.
 * 
 * <p>The resultant queue manager can then be unconfigured using the 
 * {@link examples.db2.UnConfigure} program.
 * 
 */
public class PerformanceTest {
	
    public static short[] version = { 2, 0, 0, 0 };
     
	/**
     * A definition of my queue manager.
     */
    private MQeQueueManager myQueueManager = null;

    /**
     * How long to wait for the queue manager to close cleanly, before it is closed by force ?
     * <p>Using 1 second for this example.
     */
    private static final long QUIESCE_TIME = (long) 10;
	
	/**
	 * Indicates whether this program puts MQe trace to stdout or not.
	 */
	private static final boolean MQE_TRACE = false ;
	
	private static final long NO_CONFIRM_ID = 0L; 
	private static final long TEST_CONFIRM_ID = 1L;
	
	private static final String TEST_QUEUE_NAME_DISK = "disk_queue";
	private static final String TEST_QUEUE_NAME_MEMORY = "memory_queue";
	private static final String TEST_QUEUE_NAME_DB2	 = "db2_queue";
	
	private static final String ADAPTER_NAME_DISK = "com.ibm.mqe.adapters.MQeDiskFieldsAdapter";
	private static final String ADAPTER_NAME_MEMORY = "com.ibm.mqe.adapters.MQeMemoryFieldsAdapter";
	private static final String ADAPTER_NAME_DB2  = "com.ibm.mqe.adapters.MQeJDBCFieldsAdapter";

	private static final int INDEX_DB2 = 0 ;
	private static final int INDEX_DISK = 1 ;
	private static final int INDEX_MEMORY = 2 ;
	private static final int Q_TYPES_TESTED = 3;

	private static final String[] qTypes = new String[] { "DB2   " , "Disk  " , "Memory" };
	
	private static final int MESSAGES_PUT_DURING_TEST = 1000 ;
		
    /**
     * Entry point from the command line.
     * 
     * <p>This method will perform the following operations:
     * <ul>
     * <li>Create an instance of the MQeMemoryQM class.
     * <li>Define the queue manager in memory.
     * <li>Start the queue manager.
     * <li>Put a simple message to a queue on the queue manager.
     * <li>Get the message off the queue.
     * <li>Stop the queue manager
     * <li>Remove the queue manager definition from memory.
     * <li>Optionally trace using a MQe trace handler to stdout, depending on whether a constant is set or not.
     * </ul>
     */
    public static void main(String[] args) {		
        try {			
            PerformanceTest instance = new PerformanceTest();
			instance.doWork( args );

        } catch (Exception error) {
            System.err.println("Error: " + error.toString());
            error.printStackTrace();
        }
    }

	/**
	 * An object holding all the configuration information used throughout this program.
	 */
    private ConfigInfo config ;


    
    /**
     * Does all the work in sequence, using an instance of this class.
     */
    public void doWork( String[] args ) throws Exception {
    				
		TraceController traceHelper = new TraceController();
		if( MQE_TRACE ) {
			traceHelper.traceOn();
		}

		// Get an object containing all our configuration parameters,
		// so it can be passed around to activity methods.
		config = new ConfigInfo();
		config.activate();

		// Start the queue manager
        start();
        
        try {


			String qName[] = new String[Q_TYPES_TESTED];
			
			// qName[INDEX_DB2] = config.getTestQueueName();

			MQeQueueManager qMgr = MQeQueueManager.getDefaultQueueManager();
			qName[INDEX_DB2] = createQueue( config , qMgr , TEST_QUEUE_NAME_DB2 , ADAPTER_NAME_DB2 );
			qName[INDEX_DISK] = createQueue( config , qMgr , TEST_QUEUE_NAME_DISK , ADAPTER_NAME_DISK );
			qName[INDEX_MEMORY]= createQueue( config , qMgr , TEST_QUEUE_NAME_MEMORY , ADAPTER_NAME_MEMORY );
									
			float messageRates[][] = new float[Q_TYPES_TESTED][TIMINGS_TAKEN];
			
			System.out.println();
			System.out.println();

			
			// Set the column titles for out tables.
			String columnTitle[] = new String[] { 
				"Adapter" ,"Put unassured","Get unassured","Put+Get unassured","Put assured","Get assured","Put+Get assured" };
											
			String[][] timingTableCells = new String[Q_TYPES_TESTED][TIMINGS_TAKEN+1];
			String[][] msgRateTableCells = new String[Q_TYPES_TESTED][TIMINGS_TAKEN+1];
			
			for( int qType=0 ; qType<Q_TYPES_TESTED; qType+=1 ) {
				
				long timings[] = doTimedWork( qName[qType] );
				timingTableCells[qType][0] = qTypes[qType];
				msgRateTableCells[qType][0] = qTypes[qType];
				
				for( int timingIndex = 0 ; timingIndex<TIMINGS_TAKEN; timingIndex+=1 ) {
					
					timingTableCells[qType][timingIndex+1] = Long.toString(timings[timingIndex]);
					
					if( timings[timingIndex] == 0 ) {
						msgRateTableCells[qType][timingIndex+1] = "infinite";
					} else {
						msgRateTableCells[qType][timingIndex+1] = Float.toString( (float)(1000 * MESSAGES_PUT_DURING_TEST / timings[timingIndex]) );
					}
				}																
			}
			printTable( System.out 
					  , "Milli-seconds taken for various operations using various adapters"
					  , columnTitle
					  , timingTableCells
					  );
			
			// print the table header.
			printTable( System.out
					  , "Messages per second rates for various operations using various adapters"
					  , columnTitle
					  , msgRateTableCells
					  );
					  
			deleteQueue( config , qMgr , TEST_QUEUE_NAME_MEMORY , ADAPTER_NAME_MEMORY );
			deleteQueue( config , qMgr , TEST_QUEUE_NAME_DISK , ADAPTER_NAME_DISK );
			deleteQueue( config , qMgr , TEST_QUEUE_NAME_DB2 , ADAPTER_NAME_DB2 );			
			 
        } finally {
          	stop();
        }            
            
        if( MQE_TRACE ) {
 			traceHelper.traceOff();
	    }		
    }
    
    String pad( String input , int width ) {
    	String result = "" ;
    	if( input != null ) {
    		result = input ;    	
    		while( result.length() < width ) {
	    		result = " "+result ;
    		}
    	}
    	return result;
    }
	
	
	/**
	 * Creates a disk queue. 
	 * @return The name of the queue created.
	 */
	private String createQueue( ConfigInfo config , MQeQueueManager qMgr , String qName , String adapterName ) throws Exception {
		System.out.println("Creating queue "+qName);
		AdminHelperReporter reporter = new AdminHelperReporter();
		AdminHelperDispatcher dispatcher = new AdminHelperDispatcher(qMgr);
		AdminHelperLocalQueue creationTool = new AdminHelperLocalQueue( reporter , dispatcher );
		boolean exists = creationTool.doesExist(qMgr.getName() , qMgr.getName() , qName );
		if( exists ) {
			System.out.println("Queue "+qName+" exists already. Not creating it again.");
		} else {
			String fileSeparator = MQe.fileSeparator();
			String messageDirectory = config.getBaseDirectoryName()+fileSeparator+"Queues"+fileSeparator+qName;
			creationTool.create( qMgr.getName() , qName , adapterName+":"+messageDirectory);
			System.out.println("Created queue "+qName);
		}			
		return qName ;		
	}
	
	private void deleteQueue( ConfigInfo config , MQeQueueManager qMgr , String qName , String adapterName ) throws Exception {
		System.out.println("Deleting queue "+qName);
		AdminHelperReporter reporter = new AdminHelperReporter();
		AdminHelperDispatcher dispatcher = new AdminHelperDispatcher(qMgr);
		AdminHelperLocalQueue helper = new AdminHelperLocalQueue( reporter , dispatcher );
		boolean exists = helper.doesExist(qMgr.getName() , qMgr.getName() , qName );
		if( ! exists ) {
			System.out.println("Queue "+qName+" does not exists. No need to delete it again.");
		} else {
			helper.delete( qMgr.getName() , qName );
			System.out.println("Deleted queue "+qName);
		}					
	} 
	private static final int TIMING_UNASSURED_PUT = 0 ;
	private static final int TIMING_UNASSURED_GET = 1 ;
	private static final int TIMING_UNASSURED_ROUND_TRIP = 2 ;
	private static final int TIMING_ASSURED_PUT = 3;
	private static final int TIMING_ASSURED_GET = 4;
	private static final int TIMING_ASSURED_ROUND_TRIP = 5;
	private static final int TIMINGS_TAKEN = 6;
	/**
	 * Do some work, and time how long it takes.
	 */
	private long[] doTimedWork( String qName ) throws Exception {

		System.out.println("Doing tests using queue "+qName+". This may take a while. Please be patient...");

		// Empty the queue before we start... to clean-up any existing messages.
		emptyQueue( qName );
		
		long timings[] = new long[TIMINGS_TAKEN];
				
		timings[TIMING_UNASSURED_PUT] = putMessages( qName , NO_CONFIRM_ID );		
		timings[TIMING_UNASSURED_GET] = getMessages( qName , NO_CONFIRM_ID );
		timings[TIMING_UNASSURED_ROUND_TRIP] = timings[TIMING_UNASSURED_PUT] + timings[TIMING_UNASSURED_GET] ;
		
		// Empty the queue before we start again.
		emptyQueue( qName );
		
		timings[TIMING_ASSURED_PUT] = putMessages( qName , TEST_CONFIRM_ID );
		timings[TIMING_ASSURED_GET] = getMessages( qName , TEST_CONFIRM_ID );
		timings[TIMING_ASSURED_ROUND_TRIP] = timings[TIMING_ASSURED_PUT] + timings[TIMING_ASSURED_GET] ;
		
		System.out.println("Tests using queue "+qName+" are now complete.");
		return timings;				
	}		
		
	private long putMessages( String targetQueueName , long confirmId) throws Exception {
		MQeMsgObject[] messages = prepareMessages();
		String myQueueManagerName = config.getQueueManagerName();
		long timeTaken ;
		long timeStarted = System.currentTimeMillis();
		for( int i=0;i<messages.length; i+=1 ) {
			myQueueManager.putMessage(myQueueManagerName, targetQueueName, messages[i], null, confirmId);
			if( confirmId != NO_CONFIRM_ID ) {
				myQueueManager.confirmPutMessage( myQueueManagerName , targetQueueName,messages[i].getMsgUIDFields());
			}
		}		
		long timeEnded = System.currentTimeMillis();
		timeTaken = timeEnded - timeStarted ;		
		return timeTaken;		
	}
	
	private long getMessages( String targetQueueName , long confirmId ) throws Exception {
		MQeMsgObject[] messages = new MQeMsgObject[MESSAGES_PUT_DURING_TEST];
		String myQueueManagerName = config.getQueueManagerName();
		long timeTaken ;
		long timeStarted = System.currentTimeMillis();
		for( int i=0;i<messages.length; i+=1 ) {
			messages[i] = myQueueManager.getMessage(myQueueManagerName , targetQueueName, null , null, confirmId);
			if( confirmId != NO_CONFIRM_ID ) {
				myQueueManager.confirmGetMessage( myQueueManagerName , targetQueueName,messages[i].getMsgUIDFields());
			}
		}
		long timeEnded = System.currentTimeMillis();
		timeTaken = timeEnded - timeStarted ;		
		validateMessages( messages );
		

		return timeTaken;		
	}
		
	private MQeMsgObject[] prepareMessages() throws Exception {
		// Prepare loads of messages to put.
		MQeMsgObject messages[] = new MQeMsgObject[MESSAGES_PUT_DURING_TEST];
		for( int i = 0 ; i < MESSAGES_PUT_DURING_TEST ; i +=1 ) {
			messages[i] = new MQeMsgObject();
			
			// Add my hello world text to the message.
        	messages[i].putUnicode("myFieldName", "Hello World! "+i);
		}
		return messages ;
	}			
	
	/**
	 * Checks that the messages got are the ones we sent.
	 */
	private void validateMessages( MQeMsgObject[] messages ) throws Exception {
		
		if( messages.length != MESSAGES_PUT_DURING_TEST ) {
			throw new Exception("Wrong number of messages got. Should be "
								+MESSAGES_PUT_DURING_TEST+" actually got "
								+messages.length);	
		}
		
		for( int i=0;i<messages.length ; i +=1 ) {
			MQeMsgObject msg = messages[i];
			if (msg == null) {
				throw new Exception("Message should not be null !");
			} else {
           		if (! msg.contains("myFieldName")) {
           			throw new Exception("Message should contain field 'myFieldName'");
           		} else {
               		String textGot = msg.getUnicode("myFieldName");
               		String expectedFieldValue = "Hello World! "+i;
               		if( ! textGot.equals(expectedFieldValue) ) { 
               			throw new Exception("Expected field value "+expectedFieldValue+" got value "+textGot);
               		} else {
               			// This one is valid.
               		}
           		}       		               		
			}
       	}
	}
	
	/**
	 * Get all the messages on the target queue until there are none left to get.
	 */
	private void emptyQueue( String targetQueueName ) throws Exception {
		try {
			while(true) {
				MQeMsgObject messageToForget = myQueueManager.getMessage(config.getQueueManagerName() , targetQueueName, null , null, 0L);
			}
		} catch ( MQeException e ) {
			if( e.code() == MQeExceptionCodes.Except_NotFound ) {
				// Suppress message not found exception. We've emptied the queue.
			}
		}
	}
	
    /**
     * Starts the queue manager.
     */	
    public void start() throws Exception {      
	    System.out.println("Starting the queue manager");     	
        myQueueManager = new MQeQueueManager();
        myQueueManager.activate(config.getConfigFields());
        System.out.println("Queue manager started.");
    }
	
    /**
     * Stops the queue manager.
     */
    public void stop() throws Exception {
        //System.out.println("Stopping the queue manager.");
        myQueueManager.closeQuiesce(QUIESCE_TIME);
        myQueueManager = null;
        System.out.println("Queue manager stopped.");
    }
	
	
	/**
	 * @param cells This is an array of arrays of strings. [row][column]
	 * Null strings are allowed.
	 */
	private void printTable( PrintStream out , String title , String columnTitles[] , String[][] cells ) {
		
		out.println(title);
		
		// Work out how wide each column should be.
		int columns = columnTitles.length;
		int columnWidth[] = new int[columns];
		int rows = cells.length;			
		
		// Traverse columns at a time.	
		for( int column = 0 ; column < columns	; column +=1 ) {
			String columnTitle = columnTitles[column];
			int maxWidth = 0 ;
			if( columnTitle != null ) {
				maxWidth = columnTitle.length() ;
			}
			for( int row = 0 ; row < rows ; row +=1 ) {
				int cellLength = 0;
				String cellValue = cells[row][column];
				if( cellValue != null ) {
					cellLength = cellValue.length();						
					if( cellLength > maxWidth ) {
						maxWidth = cellLength ;
					}
				}
			}
			columnWidth[column] = maxWidth;
		}
		
		
		
		// Output the column titles.
		for( int column = 0 ; column < columns ; column +=1 ) {
			out.print( pad(columnTitles[column],columnWidth[column])+", " );
		}
		out.println();
	
	
		// Output the cell data.	
		// Traverse rows at a time.
		for( int row=0 ; row < rows ; row +=1 ) {
			for( int column = 0 ; column < columns ; column +=1 ) {
				out.print( pad(cells[row][column],columnWidth[column])+", " );
			}
			out.println();
		}	
		out.println();		
	}
}
