// Version 1.1.1.1 mqe_jdbc_adapter_supportpac/source/examples/db2/Run.java, MQeDB2Adapter, la202, la202-L050712

/*-----------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms               *
 *                                                                             *
 * Licensed Materials - Property of IBM                                        *
 *                                                                             *
 * 5765-E63                                                                    *
 *                                                                             *
 * (C) Copyright IBM Corp. 2002                                                *
 *-----------------------------------------------------------------------------*/
package examples.db2;


import com.ibm.mqe.*;
import com.ibm.mqe.adapters.*;
import com.ibm.mqe.registry.*;
import com.ibm.mqe.trace.MQeTraceToReadable;
import java.util.*;
import examples.mqbridge.administration.programming.*;


/**
 * Uses the configured queue manager previously defined by the {@link examples.db2.Configure} program.
 * 
 * <p>A message is put to the test queue, and got from the test queue.
 * 
 * <p>The resultant queue manager can then be unconfigured using the 
 * {@link examples.db2.UnConfigure} program.
 * 
 */
public class Run {
	
    public static short[] version = { 2, 0, 0, 0 };
    
	/**
     * A definition of my queue manager.
     */
    private MQeQueueManager myQueueManager = null;

    /**
     * How long to wait for the queue manager to close cleanly, before it is closed by force ?
     * <p>Using 1 second for this example.
     */
    private static final long QUIESCE_TIME = (long) 1000;
	
	/**
	 * Indicates whether this program puts MQe trace to stdout or not.
	 */
	private static final boolean MQE_TRACE = false ;

    /**
     * Entry point from the command line.
     * 
     * <p>This method will perform the following operations:
     * <ul>
     * <li>Create an instance of the MQeMemoryQM class.
     * <li>Define the queue manager in memory.
     * <li>Start the queue manager.
     * <li>Put a simple message to a queue on the queue manager.
     * <li>Get the message off the queue.
     * <li>Stop the queue manager
     * <li>Remove the queue manager definition from memory.
     * <li>Optionally trace using a MQe trace handler to stdout, depending on whether a constant is set or not.
     * </ul>
     */
    public static void main(String[] args) {		
        try {			
            Run instance = new Run();
			instance.doWork( args );

        } catch (Exception error) {
            System.err.println("Error: " + error.toString());
            error.printStackTrace();
        }
    }

	/**
	 * An object holding all the configuration information used throughout this program.
	 */
    private ConfigInfo config ;
    
    /**
     * Does all the work in sequence, using an instance of this class.
     */
    public void doWork( String[] args ) throws Exception {
    				
		TraceController traceHelper = new TraceController();
		if( MQE_TRACE ) {
			traceHelper.traceOn();
		}

		// Get an object containing all our configuration parameters,
		// so it can be passed around to activity methods.
		config = new ConfigInfo();
		config.activate();

		// Start the queue manager
        start();
        
        try {

			String qName = config.getTestQueueName();
			
			for( int i=0; i<1000; i+=1 ) {
				put( qName );
				get( qName );
			
				putAssured( qName );
				getAssured( qName );
			}
			
        } finally {
          	stop();
        }            
            
        if( MQE_TRACE ) {
 			traceHelper.traceOff();
	    }		
    }
	
    /**
     * Starts the queue manager.
     */	
    public void start() throws Exception {      
	    System.out.println("Starting the queue manager");     	
        myQueueManager = new MQeQueueManager();
        myQueueManager.activate(config.getConfigFields());
        //System.out.println("Queue manager started.");
    }
	
    /**
     * Puts a simple message to the system default local queue.
     */
    public void put( String targetQueueName ) throws Exception {

	    //System.out.println("Putting the test message to queue "+targetQueueName+" with a zero confirm id.");
       		
        MQeMsgObject msg = new MQeMsgObject();

        // Add my hello world text to the message.
        msg.putUnicode("myFieldName", "Hello World! Target Queue="+targetQueueName);
        
        myQueueManager.putMessage(config.getQueueManagerName() , targetQueueName, msg, null, 0L);
        
	    //System.out.println("Put the test message to queue "+targetQueueName+" ok.");       
    }
    
    /**
     * Puts a simple message to the system default local queue.
     */
    public void putAssured( String targetQueueName ) throws Exception {
    	//System.out.println("Putting the test message to queue "+targetQueueName+" using a non-zero confirm id.");
		
        MQeMsgObject msg = new MQeMsgObject();

        // Add my hello world text to the message.
        msg.putUnicode("myFieldName", "Hello World! Target Queue="+targetQueueName);
        
        myQueueManager.putMessage(config.getQueueManagerName(), targetQueueName, msg, null, 51L);
        myQueueManager.confirmPutMessage( config.getQueueManagerName(),targetQueueName,msg.getMsgUIDFields());
       
	    //System.out.println("Put the test message to queue "+targetQueueName+" using a non-zero confirm id ok.");
    }
	
    /**
     * Gets a simple message from the system default local queue.
     */
    public void get( String targetQueueName ) throws Exception {
	    //System.out.println("Getting the test message from queue "+targetQueueName);

        MQeMsgObject msg = myQueueManager.getMessage( config.getQueueManagerName(), targetQueueName, null, null, 0L );

        if (msg != null) {
            //System.out.println("Got the test message.");

           	if (msg.contains("myFieldName")) {
               	String textGot = msg.getUnicode("myFieldName");
                //System.out.println("Message contained the text '" + textGot + "'");
   	        }
       	}
    }
    
    /**
     * Gets a simple message from the system default local queue.
     */
    public void getAssured( String targetQueueName ) throws Exception {
    	//System.out.println("Getting the test message from queue "+targetQueueName);
     
        MQeMsgObject msg = myQueueManager.getMessage(config.getQueueManagerName(), targetQueueName, null, null, 51L );
        
        if (msg != null) {
	       	myQueueManager.confirmGetMessage( config.getQueueManagerName() , targetQueueName , msg.getMsgUIDFields() );
 
            //System.out.println("Got the test message.");
           	if (msg.contains("myFieldName")) {
               	String textGot = msg.getUnicode("myFieldName");
                //System.out.println("Message contained the text '" + textGot + "'");
   	        }
       	}
    }
	
    /**
     * Stops the queue manager.
     */
    public void stop() throws Exception {
        //System.out.println("Stopping the queue manager.");
        myQueueManager.closeQuiesce(QUIESCE_TIME);
        myQueueManager = null;
        System.out.println("Queue manager stopped.");
    }
	
}
