// Version 1.1.1.1 mqe_db2_adapter_supportpac/source/examples/db2/UnConfigure.java, MQeDB2Adapter, la202

/*-----------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms               *
 *                                                                             *
 * Licensed Materials - Property of IBM                                        *
 *                                                                             *
 * 5765-E63                                                                    *
 *                                                                             *
 * (C) Copyright IBM Corp. 2002                                                *
 *-----------------------------------------------------------------------------*/
package examples.db2 ;

import com.ibm.mqe.*;
import examples.mqbridge.administration.programming.*;

/**
 * Removes a queue manager created using the {@link examples.db2.Configure} program.
 */
public class UnConfigure {
	/**
	 * Indicates whether this program puts MQe trace to stdout or not.
	 */
	private static final boolean MQE_TRACE = false ;
	
	public static short[] version = { 2, 0, 0, 0 };
	
    public static void main(String[] args) {
        try {
        	TraceController traceHelper = new TraceController();    
    		if( MQE_TRACE ) {    	        	
        		traceHelper.traceOn();
    		}
    		
        	UnConfigure me = new UnConfigure();
        	me.deleteTestQueue();
        	me.undefine();
        	
        	if( MQE_TRACE ) {
        		traceHelper.traceOff();
        	}
        } catch (Exception e) {
            System.out.println("Failure: " + e.toString());
            e.printStackTrace();
        }	
    }
	
	/**
     * Removes any definitions on the queue manager which were previously defined.
     */
    public void undefine() throws Exception {		    	
		
        // Create all the configuration information needed to construct the 
        // queue manager
        ConfigInfo info = new ConfigInfo();
        info.activate();
        
        MQeFields configFields = info.getConfigFields(); 
        String queueManagerName = info.getQueueManagerName();
        String systemQueueSorageLocation = info.getSystemQueueStorageLocation();
         
        System.out.println("Queue manager " + queueManagerName + " will now be deleted.");
		
        MQeQueueManagerConfigure configurator = new MQeQueueManagerConfigure();

        configurator.activate(configFields, systemQueueSorageLocation);
		
        // Remove the configured entries for the queues we defined.
        configurator.deleteSystemQueueDefinition();
        configurator.deleteDeadLetterQueueDefinition();
        configurator.deleteAdminReplyQueueDefinition();
        configurator.deleteAdminQueueDefinition();
		
        // Remove the queue manager itself.
        configurator.deleteQueueManagerDefinition();
		
        configurator.close();
		
        System.out.println("Queue manager is now deleted.");
    }  
    
    /**
     * Delete a test queue using administration messages, so we can put a test message to it.
     * 
     * <p>We could have used the system default local queue, but here we are showing an example
     * of how one might create a queue which has a long filename itself, but stores messages
     * on an 8.3 file sysyem. 
     * 
     * @throws Exception if there is a failure.
     */
    public void deleteTestQueue() throws Exception {
    	System.out.println("Deleting a test queue");
    	
    	ConfigInfo configInfo = new ConfigInfo();
		configInfo.activate();
		
		MQeFields configFields = configInfo.getConfigFields();
		String qStorageLocation = configInfo.getTestQueueStorageLocation();
		String queueManagerName = configInfo.getQueueManagerName();
		String fileSeparator = MQe.fileSeparator();
		String testQueueName = configInfo.getTestQueueName();

    	
        // Now fire-up the queue manager so we can adminster it using adminstration messages...
        System.out.println("Starting the queue manager so we can administer it.");
        MQeQueueManager myQueueManager = new MQeQueueManager();
        myQueueManager.activate(configFields);
        System.out.println("Queue manager started ok.");
        
        // Construct a test queue explicitly which stores 8.3 message fies in a short filename directory.
		Reporter reporter = (Reporter) new AdminHelperReporter();
		AdminMessageDispatcher dispatcher = (AdminMessageDispatcher)new AdminHelperDispatcher(myQueueManager);
		AdminHelperLocalQueue helper = new AdminHelperLocalQueue( reporter , dispatcher );
		  
		try {		  
	        try {
	        	// Drain all the messages out of the test queue before we remove it.
	        	boolean drained = false ;        	
	        	while( !drained ) {
	        		MQeMsgObject msg = dispatcher.get( reporter , queueManagerName , testQueueName );
	        		if( msg == null ) {
	        			drained = true ;
	        		}
	        	}
	        	
	        	System.out.println("Test queue "+testQueueName+" now drained.");
	        	
	        	// Now that the test queue is empty, remove it.
	        	System.out.println("Deleting queue "+testQueueName);
				helper.delete(  queueManagerName , testQueueName );
				System.out.println("Deleted queue "+testQueueName+" ok.");
	        } catch( Exception e ) {
	        	reporter.error("Failed to delete the test queue.");        	
	        	reporter.error(e);
	        }
	        	       
        } finally {
        	// Even if we failed, make sure the queue manager is closed down within 5 seconds.
        	System.out.println("Closing the queue manager.");
        	myQueueManager.closeQuiesce(5000L);
        	System.out.println("Closed the queue manager ok,");
        }		
    }	     
}

